/* mod2zzt.h - Basic data structures for working with .mod files
 * Copyright (C) 2007 Chris Mounce. */
#ifndef _MOD2ZZT_H_
#define _MOD2ZZT_H_

/*
 *	Structures
 */

/* A .mod pattern */
typedef struct {
	/* Each char represents a note, according to the following table:
	 *  -1        =  Continue previous note
	 *	0         =  Rest
	 *  1 to 12   =  C, C#, D ... B, all 2 octaves down
	 *  13 to 24  =  C, C#, D ... B, all 1 octave down
	 *  25 to 36  =  C, C#, D ... B, all on the default octave
	 *  37 to 48  =  C, C#, D ... B, all 1 octave up
	 *  49 to 60  =  C, C#, D ... B, all 2 octaves up
	 */
	char primary[64]; /* notes from primary channel */
	char secondary[64]; /* notes from secondary channel */
} pattern;

/* Everything we need to know about a .mod file */
typedef struct {
	char numChannels;
	char numSamples;
	char sampleVolumes[31];
	unsigned char numPatterns;
	unsigned char patternTable[128];
	unsigned char patternTableSize;
	pattern* patterns[128];
} modInfo;

/* A single stream of PC speaker beeps */
typedef struct {
	unsigned int size;
	char notes[256]; /* Each note is 1/20 second long */
} ZZTmusic;



/*
 *	Tables, enumerations, and similar stuff
 */

/* Playback speeds */
enum { SLOW, NORMAL, FAST };

/* Table for converting periods to notes */
const int NOTE_MIDPOINTS[] = {
/* C     C#     D     D#     E     F     F#     G     G#     A     A#     B     */
    1664,  1570, 1482,  1398, 1319, 1245,  1175, 1109,  1047, 989,   934,  881, 
    832,   785,  741,   699,  659,  622,   587,  554,   523,  494,   466,  440, 
    416,   392,  370,   349,  329,  311,   293,  277,   261,  247,   233,  220, 
    208,   196,  185,   175,  165,  155,   147,  139,   131,  123,   116,  110, 
    104,   98,   92,    87,   82,   78,    73,   69,    65,   62,    58,   0
/* C     C#     D     D#     E     F     F#     G     G#     A     A#     B     */
};

/* Codes for each of the notes in an octave */
const char* NOTE_CODES[] = {"c", "c#", "d", "d#", "e", "f", "f#", "g", "g#", "a", "a#", "b"};

/* Durations of notes that are easy to produce (1 = 1/32 beat) */
const unsigned char EASY_DURATIONS[] = {108, 72, 54, 48, 36, 32, 27, 24, 18, 16, 12, 9, 8, 6, 4, 3, 2, 1};

/* Codes used to get these easy durations */
const char* DURATION_CODES[] = {"w...", "w..", "h...", "w.", "h..", "w", "q...", "h.", "q..", "h", "q.", "i..", "q", "i.", "i", "s.", "s", "t"};



/*
 * Function prototypes
 */

int main(int argc, char* argv[]);
void printUsage();

void readModInfo(FILE* input, modInfo* mod);
void readModPatterns(FILE* input, modInfo* mod, char primaryChannel, char secondaryChannel);
int isMagic(char* s1, char* s2);

void convertPattern1(pattern* p, char speed);
void convertPattern2(pattern* p, char speed);
void convertPattern(ZZTmusic* notes);
void writeNote(char note, unsigned char duration);

void error(char* message);

#endif